# Dynamic Archive
Dynamic Archive is a Object-Relational Mapping system written in Ruby. It facilitates
storage of complex programming objects in databases. Information about objects are
stored in tables and retrieved from the database as needed and re-instantiated as
Ruby objects. Custom SQL queries allows for elaborate relationships between objects
to easily be formed.

The core of Dynamic Archive lives in [lib](./lib). Dynamic Archive plays a major role
as part of [Ruby on Trails](https://github.com/wmcmeans/trails).

## [DynamicArchive::Base](./lib/base.rb)

The `Base` class is the core of Dynamic Archive. The class creates attribute methods based
on column names in the `::finalize` method. This way, all classes that inherit will have
instance methods for properties, e.g. how you can call something like `User#name`
(and also `User#name=`).

Base also has some SQL query methods, like `::find` that take an ID and return the first
result from the database (or nil). The `#save` method nicely packages up the `update`
and `insert` methods as one, even though the two require two different query strings.

## Associatable, Searchable, Validatable

These modules are extended to the `Base` class and make it more powerful.

### [DynamicArchive::Associatable](./lib/associatable.rb)
`Associatable` allows rich relationships to be formed between objects of different classes.
Its `::has_many`,`::belongs_to`, `::has_one_through`, and `::has_many_through` methods
can generate a complex mapping of interelation.

### [DynamicArchive::Searchable](./lib/searchable.rb)
As the name suggests, `Searchable` gives extra SQL searching capabilities most notably
through its `::where` method that allows querying using an interpolated query string or
with a hash of params.

### [DynamicArchive::Validatable](./lib/validatable.rb)
`Validatable` gives the option of adding validations to class attributes.
`::validates` adds a list of attributes to be validated followed by a hash of validations
to check for each. Basic validations such as `length`, `presence`, and `uniqueness`
are included.
