module DynamicArchive
  module Validatable
    def validates(*args)
      validated_attrs = []
      options = {}
      args.each do |arg|
        if arg.is_a?(Hash)
          options = arg
        else
          validated_attrs << arg
        end
      end

      validated_attrs.each do |attr|
        validations[attr] = validations[attr].merge(options)
      end
    end

    def is_valid?(instance)
      instance.errors.clear
      valid = true
      columns.each do |attr|
        validations[attr].each do |validation, value|
          validated = send(validation, instance, attr, value)
          unless validated
            instance.errors << "#{attr} did not pass #{validation} validation"
            valid = false
          end
        end
      end

      valid
    end

    def length(instance, attr, value)
      min = value[:minimum] || 0
      max = value[:maximum]
      if max
        instance.send(attr).length.between?(min, max)
      else
        instance.send(attr).length >= min
      end
    end

    def presence(instance, attr, value)
      if value
        !!instance.send(attr)
      else
        !instance.send(attr)
      end
    end

    def uniqueness(instance, attr, value)
      if value == true
        if instance.id
          return !instance.class
            .where("#{attr} = (?) AND id != (?)", instance.send(attr), instance.id).first
        else
          return !instance.class
            .where({ attr => instance.send(attr) }).first
        end
      end

      true
    end

    def validations
      @validations ||= Hash.new { |h, k| h[k] = {} }
    end
  end
end
