require 'validatable'

describe 'Validatable' do
  before(:each) { DBConnection.reset }
  after(:each) { DBConnection.reset }

  before(:all) do
    class Cat < DynamicArchive::Base
      finalize!
    end

    class Human < DynamicArchive::Base
      self.table_name = 'humans'
      validates :fname, :lname, presence: true

      finalize!
    end
  end

  it '::validations returns a hash of hashes' do
    expect(Cat.validations).to be_a(Hash)
    expect(Cat.validations[:name]).to eq({})
  end

  it '::validations returns any validations added' do
    expect(Human.validations[:fname]).to eq({ presence: true })
  end

  it '#save calls validations' do
    alex = Human.new(fname: "Alex")
    expect(Human).to receive(:presence).twice
    alex.save
  end

  it "doesn't save invalid instances" do
    alex = Human.new(fname: "Alex")
    alex.save
    expect(alex.errors[0]).to eq("lname did not pass presence validation")
    expect(Human.all.last.fname).to_not eq("Alex")
  end

  it "#uniqueness does not prevent updating attributes" do
    class Human
      validates :fname, uniqueness: true
    end

    alex = Human.new(fname: "Alex", lname: "Rodriguez")
    alex.save
    alex.lname = "The Great"
    alex.save

    expect(alex.errors[0]).to be_nil
  end

  it "#uniqueness doesn't allow two entries with same value if true" do
    class Human
      validates :fname, uniqueness: true
    end

    alex1 = Human.new(fname: "Alex", lname: "Rodriguez")
    alex1.save

    alex2 = Human.new(fname: "Alex", lname: "de Ronde")
    alex2.save
    expect(alex2.errors[0]).to eq("fname did not pass uniqueness validation")
  end

  it "#uniqueness does not add flag unconstrained attributes" do
    class Human
      validates :fname, uniqueness: true
    end

    alex = Human.new(fname: "Alex", lname: "Rodriguez")
    alex.save
    natalie = Human.new(fname: "Natalie", lname: "Lin")
    natalie.save

    expect(natalie.errors[0]).to be_nil
    expect(alex.errors[0]).to be_nil
  end
end
