require 'active_support'
require 'active_support/core_ext'
require 'active_support/inflector'
require 'erb'
require_relative './session'
require_relative './flash'

module TrailsController
  class Base
    attr_reader :req, :res, :params

    def initialize(req, res, route_params = {})
      @req = req
      @res = res
      @params = req.params.merge(route_params)
      @already_built_response = false
    end

    def already_built_response?
      @already_built_response
    end

    def flash
      @flash ||= Flash.new(@req)
    end

    def redirect_to(url)
      raise 'Response already built' if already_built_response?

      res.status = 302
      res['Location'] = url
      session.store_session(res)
      flash.store_flash(res)

      @already_built_response = true
    end

    def render_content(content, content_type)
      raise 'Cannot render twice' if already_built_response?

      res['Content-Type'] = content_type
      res.write(content)
      session.store_session(res)
      flash.store_flash(res)

      @already_built_response = true
    end

    def render_partial(params = {})
      defaults = {
        view_dir: self.class.name.underscore[0..-12],
        template: @__template_name
      }
      params = defaults.merge(params)

      file_path = "app/views/#{params[:view_dir]}/#{params[:template]}.html.erb"
      file_contents = File.read(file_path)
      ERB.new(file_contents).result(binding)
    end

    def render(template_name)
      app_template = File.read("app/views/layouts/application.html.erb")
      @__template_name = template_name

      result = ERB.new(app_template).result(binding)
      render_content(result, 'text/html')
    end

    def session
      @session ||= Session.new(@req)
    end

    def invoke_action(name)
      send(name)
      render(name) unless already_built_response?
    end
  end
end
